/**
 * Bundle of: vue-class-store
 * Generated: 2020-05-22
 * Version: 2.0.5
 */

import Vue from 'vue';

function makeOptions(model) {
    // prototype
    var prototype = Object.getPrototypeOf(model);
    if (!prototype || prototype === Object.prototype) {
        return {};
    }
    // parent options
    var extendsOptions = makeOptions(prototype);
    // descriptors
    var descriptors = Object.getOwnPropertyDescriptors(prototype);
    // options
    var name = prototype.constructor.name;
    var data = {};
    var computed = {};
    var methods = {};
    var watch = {};
    // data, string watches
    Object.keys(model).forEach(function (key) {
        var value = model[key];
        if (key.startsWith('on:')) {
            watch[key.substring(3)] = value;
        }
        else {
            data[key] = value;
        }
    });
    // function watches, methods, computed
    Object.keys(descriptors).forEach(function (key) {
        if (key !== 'constructor' && !key.startsWith('__')) {
            var ref = descriptors[key];
            var value = ref.value;
            var get = ref.get;
            var set = ref.set;
            if (key.startsWith('on:')) {
                watch[key.substring(3)] = value;
            }
            else if (value) {
                methods[key] = value;
            }
            else if (get && set) {
                computed[key] = { get: get, set: set };
            }
            else if (get) {
                computed[key] = get;
            }
        }
    });
    // return
    return {
        name: name,
        extends: extendsOptions,
        computed: computed,
        methods: methods,
        watch: watch,
        data: data,
    };
}
function makeVue(model) {
    var options = makeOptions(model);
    return new Vue(options);
}
function VueStore(constructor) {
    function construct() {
        var args = [], len = arguments.length;
        while ( len-- ) args[ len ] = arguments[ len ];

        var instance = new (Function.prototype.bind.apply( constructor, [ null ].concat( args) ));
        return makeVue(instance);
    }
    return construct;
}
VueStore.create = makeVue;

export default VueStore;
export { makeOptions, makeVue };
